@@ -17,19 +17,30 @@ import pandas as pd
 import os
 import subprocess
 import atexit
+
+
 from logic_utils import pd_dataframe_to_m_expression, pandas_datatype_to_tabular_datatype, ticks_to_datetime, remove_suffix
 from tabular_tracing import Refresh_Trace
 
 
 class Tabular:
-	'''Tabular Class to perform operations: [Microsoft.AnalysisServices.Tabular](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular?view=analysisservices-dotnet)
+	'''Tabular Class to perform operations: [Microsoft.AnalysisServices.Tabular](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular?view=analysisservices-dotnet). You can use this class as your main way to interact with your model.
 
 	Args:
 		CONNECTION_STR (str): Valid [Connection String](https://docs.microsoft.com/en-us/analysis-services/instances/connection-string-properties-analysis-services?view=asallproducts-allversions) for connecting to a Tabular Model.
+	Attributes:
+		Server (Server): See [Server MS Docs](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.server?view=analysisservices-dotnet).
+		Catalog (str): Name of Database. See [Catalog MS Docs](https://learn.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.connectioninfo.catalog?view=analysisservices-dotnet#microsoft-analysisservices-connectioninfo-catalog).
+		Model (Model): See [Model MS Docs](https://learn.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular.model?view=analysisservices-dotnet).
+		AdomdConnection (AdomdConnection): For querying. See [AdomdConnection MS Docs](https://learn.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.adomdclient.adomdconnection?view=analysisservices-dotnet). Connection made from parts of the originally provided connection string.
+		Tables (List[Table]): Easy access list of tables from model. See [Table MS Docs](https://learn.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular.table?view=analysisservices-dotnet).
+		Columns (List[Column]): Easy access list of columns from model. See [Column MS Docs](https://learn.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular.column?view=analysisservices-dotnet).
+		Partitions (List[Partition]): Easy access list of partitions from model. See [Partition MS Docs](https://learn.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.partition?view=analysisservices-dotnet).
+		Measures (List[Measure]): Easy access list of measures from model. See [Measure MS Docs](https://learn.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular.table.measures?view=analysisservices-dotnet#microsoft-analysisservices-tabular-table-measures).
 	'''	
 	def __init__(self,CONNECTION_STR:str):
 		logger.debug(f'Initializing Tabular Class')
-		self.Server = Server() #[Server](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.server?view=analysisservices-dotnet)
+		self.Server = Server()
 		self.Server.Connect(CONNECTION_STR)
 		logger.info(f'Connected to Server - {self.Server.Name}')
 		self.Catalog = self.Server.ConnectionInfo.Catalog
@@ -37,14 +48,16 @@ class Tabular:
 		try:
 			self.Database = [database for database in self.Server.Databases.GetEnumerator() if database.Name == self.Catalog][0]
 		except:
-			logger.error(f'Unable to find Database... {self.Catalog}')
+			err_msg = f'Unable to find Database... {self.Catalog}'
+			logger.error(err_msg)
+			raise Exception(err_msg)
 		logger.info(f'Connected to Database - {self.Database.Name}')
 		self.CompatibilityLevel: int = self.Database.CompatibilityLevel
 		self.CompatibilityMode: int = self.Database.CompatibilityMode.value__
 		self.Model = self.Database.Model
 		logger.info(f'Connected to Model - {self.Model.Name}')
-		self.DaxConnection = AdomdConnection()
-		self.DaxConnection.ConnectionString = f"{self.Server.ConnectionString}Password='{self.Server.ConnectionInfo.Password}'"
+		self.AdomdConnection = AdomdConnection()
+		self.AdomdConnection.ConnectionString = f"{self.Server.ConnectionString}Password='{self.Server.ConnectionInfo.Password}'"
 		self.Reload_Model_Info()
 		logger.debug(f'Class Initialization Completed')
 		logger.debug(f'Registering Disconnect on Termination...')
@@ -65,6 +78,14 @@ class Tabular:
 		self.Measures = [measure for table in self.Tables for measure in table.Measures.GetEnumerator()]
 		self.Database.Refresh()
 		return True
+	def Is_Process(self) -> bool:
+		'''Run method to check if Processing is occurring. Will query DMV $SYSTEM.DISCOVER_JOBS to see if any processing is happening.
+
+		Returns:
+			bool: True if DMV shows Process, False if not.
+		'''		
+		_jobs_df = self.Query("select * from $SYSTEM.DISCOVER_JOBS")
+		return True if len(_jobs_df[_jobs_df['JOB_DESCRIPTION']=='Process']) > 0 else False
 	def Disconnect(self) -> bool:
 		'''Disconnects from Model
 
@@ -99,12 +120,15 @@ class Tabular:
 		'''		
 		logger.debug(f'Beginning RequestRefresh cadence...')
 
-		def Refresh_Report(Property_Changes):
+		def _Refresh_Report(Property_Changes) -> pd.DataFrame:
 			logger.debug(f'Running Refresh Report...')
+			refresh_data = []
 			for property_change in Property_Changes:
 				if isinstance(property_change.Object,Partition) and property_change.Property_Name == 'RefreshedTime':
-					logger.info(f'{property_change.Object.Table.Name} - {property_change.Object.Name} Refreshed! - {ticks_to_datetime(property_change.New_Value.Ticks).strftime("%m/%d/%Y, %H:%M:%S")}')
-			return True
+					table, partition, refreshed_time = property_change.Object.Table.Name, property_change.Object.Name, ticks_to_datetime(property_change.New_Value.Ticks)
+					logger.info(f'{table} - {partition} Refreshed! - {refreshed_time.strftime("%m/%d/%Y, %H:%M:%S")}')
+					refresh_data += [[table, partition, refreshed_time]]
+			return pd.DataFrame(refresh_data, columns=['Table','Partition','Refreshed Time'])
 				
 		def refresh_table(table:Table) -> None:
 			logging.info(f'Requesting refresh for {table.Name}')
@@ -163,9 +187,7 @@ class Tabular:
 			rt.Stop()
 			rt.Drop()
 
-		Refresh_Report(m.Property_Changes)
-
-		return m
+		return _Refresh_Report(m.Property_Changes)
 	def Update(self, UpdateOptions:UpdateOptions = UpdateOptions.ExpandFull) -> None:
 		'''[Update Model](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.majorobject.update?view=analysisservices-dotnet#microsoft-analysisservices-majorobject-update(microsoft-analysisservices-updateoptions))
 
@@ -349,28 +371,22 @@ class Tabular:
 				Query_Str = str(file.read())
 				
 		
-		logger.debug(f'Beginning to Query...')
 		try:
-			logger.debug(f'Attempting to Open Adomd Connection...')
-			self.DaxConnection.Open()
+			logger.debug(f'Setting first initial Adomd Connection...')
+			self.AdomdConnection.Open()
 			logger.debug(f'Connected!')
 		except: 
-			logger.debug(f'Connection skipped already connected...')
 			pass
-		logger.info(f'Querying Model with Query...')
-		Query =  AdomdCommand(Query_Str, self.DaxConnection).ExecuteReader()
-		logger.debug(f'Determining Field Count...')
+		logger.info(f'Querying Model...')
+		Query =  AdomdCommand(Query_Str, self.AdomdConnection).ExecuteReader()
 		Column_Headers = [(index,Query.GetName(index)) for index in range(0,Query.FieldCount)]
 		Results = list()
-		logger.debug(f'Converting Results into List...')
 		while Query.Read():
 			Results.append([Query.GetValue(index) for index in range(0,len(Column_Headers))])
-		logger.debug(f'Data retrieved and closing query...')
 		Query.Close()
-		logger.debug(f'Converting to Pandas DataFrame...')
+		logger.debug(f'Data retrieved... reading...')
 		df = pd.DataFrame(Results,columns=[value for _,value in Column_Headers])
 		if len(df) == 1 and len(df.columns) == 1:
-			logging.debug(f'Returning single value...')
 			return df.iloc[0][df.columns[0]]
 		return df
 	def Query_Every_Column(self,query_function:str='COUNTROWS(VALUES(_))') -> pd.DataFrame:
@@ -427,10 +443,10 @@ class Tabular:
 		Returns:
 			List[str]: Assuming no failure, will return list of BPA violations. Else will return error from command line.
 		'''		
-		#Working TE2 Script in Python os.system(f"start /wait {te2.EXE_Path} \"Provider=MSOLAP;{model.DaxConnection.ConnectionString}\" FINANCE -B \"{os.getcwd()}\\Model.bim\" -A {l.BPA_LOCAL_FILE_PATH} -V/?")
+		#Working TE2 Script in Python os.system(f"start /wait {te2.EXE_Path} \"Provider=MSOLAP;{model.AdomdConnection.ConnectionString}\" FINANCE -B \"{os.getcwd()}\\Model.bim\" -A {l.BPA_LOCAL_FILE_PATH} -V/?")
 		#start /wait 
 		logger.debug(f'Beginning request to talk with TE2 & Find BPA...')
-		cmd = f"{Tabular_Editor_Exe} \"Provider=MSOLAP;{self.DaxConnection.ConnectionString}\" {self.Database.Name} -B \"{os.getcwd()}\\Model.bim\" -A {Best_Practice_Analyzer} -V/?"
+		cmd = f"{Tabular_Editor_Exe} \"Provider=MSOLAP;{self.AdomdConnection.ConnectionString}\" {self.Database.Name} -B \"{os.getcwd()}\\Model.bim\" -A {Best_Practice_Analyzer} -V/?"
 		logger.debug(f'Command Generated')
 		logger.debug(f'Submitting Command...')
 		sp = subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,universal_newlines=True)
@@ -476,4 +492,4 @@ class Tabular:
 		self.Model.Tables.Add(new_table)
 		self.Refresh([new_table])
 		self.SaveChanges()
-		return True
+		return True
\ No newline at end of file
